# ToF Release Notes

![Scheme](documentation/chronoptics.png)

## Release 3.7.0
- Overhauled viewer UI to make it more user friendly. All the previous options are still possible, they might just have been hidden away.
- Added MLX saturation detection. This will be automatically enabled when you set a new camera configuration, old camera configurations won't automatically be updated.
- Fixed issue with BGR_PROJECTED data being incorrect when xyz_dimensions was set to be in meters.
- Fixed issue in the usb interface communicating with older cameras.

## Release 3.6.4
- Viewer: Fix record data not saving images in the specified location
- Viewer: Add rotate button to 2d displays and zooming now zooms in on your cursor
- Add create_kea_camera_gige function which allows you to construct a KeaCamera with a packet size

## Release 3.6.3
- Fix camera not rebooting on sensor failure

## Release 3.6.2
- Fixed pipeline bug, when reading csf files with raw and rgb data without using the rgb data, the gpu pipeline would throw an exception

## Release 3.6.1
- Fixed KeaCamera bug, when running pipeline on CPU with on camera processing disabled it wouldn't stream

## Release 3.6.0
- Added rust support, this is still in an experimental stage, so the interface could change without major version increment.
- Added RGB sync support. There used to be a large delay (~500ms) between Depth and RGB, this is now less than (~33ms).
- Added swupdate support. Camera can now be updated using the tof library or via a web interface. The web interface can be found at the ip address of the camera and port 8080, for example http://192.168.1.208:8080

## Release 3.5.1
- Fixed windows reverting back to old file explorer
- Added full screen display to the viewer

## Release 3.5.0
- Changed viewer color mode from jet to turbo
- The export ply and image buttons export the data you're currently looking at instead of the next data to come in.
- The viewer can now record data into various mediums. Our csf files, image files (.jpeg, .pfm and .pgm), ply files and videos. These were all previously possible, but now it all has the same options as capturing csf files.
- Added RAW and COMMON data viewers.
- Cyclops: Reworked dot detection algorithm

## Release 3.4.0
- Now available for android, minimum api version is 24
- Fixed bug in viewer that would export a 100 ply files
- Added software and hardware trigger support to the camera. The trigger mode needs to be configured in the CameraConfig. IMPORTANT NOTE: If the time between triggers is larger than 250ms the camera will internally run the light source to keep it warm, it will be run at the modulation frequencies the camera is configured at.

## Release 3.3.2
- Fixed crash in viewer relating to camera and processing config
- Improved boot speeds of the camera

## Release 3.3.1
- Added discover kea cameras function to discover all available kea cameras
- Multiple ply files can now be exported

## Release 3.3.0
- Fixed ply and image export
- Massively improved EmbeddedKeaCamera BGR processing speeds by utilizing vpu
- Added XYZ Dimension select
- Added Rigid Transformation
- Improved stability of gige and usb interface
- MixedPixelLUT now only needs to be generated once per camera per frequency pair

## Release 3.2.5
- Added native file dialog to the viewer
- Fixed cyclops missing dots

## Release 3.2.4
- Fixed cyclops/thanos camera not being discoverable in viewer
- Mixed pixel now works with on camera processing

## Release 3.2.3
- Harmonic cancellation is now applied per sub frame
- Fixed bug with RAW_AB in csf files

## Release 3.2.2
- Fixed flickering in cyclops camera

## Release 3.2.1
- Fixed arm executables not being included

## Release 3.2.0
- Added python example on how to set static ip
- Updated ros example
- Fixed many bugs
- Added local means filter. It acts similar to the bilateral filter, only adds values that are within the standard deviation. But it just takes a simple mean, hence the name.
- Temporal, bilateral and local means filter use common data if available
- Added dot compression
- Added software binning
- On camera processing is now enabled by default
- Added set_configurations to most cameras
- Added get_version function to query the version of the library
- Added version function to interfaces and KeaCamera to query the library version on the camera
- Removed mixed pixel files from installer. They can now be found [here](https://chronoptics.bitbucket.io/mixed_pixel_files/index.html)
- Mixed pixel can now generate look up tables on demand, this takes several seconds but only has to be done once for the program lifetime
- Added support for 1:3 mixed pixel ratios
- Added support for three frequency mixed pixel

## Release 3.1.0
- Dot mask can now be disabled.
- Cyclops: Added new dot detection algorithm.
- Updated pipeline internals. 
    - Now any stream type can be processed by the CSFCamera, not just raw and Radial/Intensity. 
    - Output from the get_frames is now ordered according to the order that is specified in set_stream_list 
    - Frame count is the same for each frame in the frames gotten from get_frames
    - CPU and GPU pipelines output is now more closely related

## Release 3.0.2
- Added python2 generation

## Release 3.0.1
- Added channel support. This allows you to operate multiple cameras near each other as long as each one is on a different channel.
- Fixed some issues around cyclops camera
- Made some improvements to the viewer
- Reworked viewer with feedback regarding cyclops camera

## Release 3.0.0
Breaking changes:
- Removed all structs, all data is now accessed using functions
- Reworked and renamed ToFFrame to tof_data_t
- Removed ToFFrameHeader and moved functionality to tof_data_t
- Reworked and renamed ToFStream to tof_stream_t
- Reworked and renamed ToFProcessingConfig to tof_processing_config_t
- Creation of CsfWriter has changed
- Reworked CsfReader. It now uses streams instead of types and there's an easy way to just iterate over all the frames.
- C: handling of dynamic arrays (vector/list) is now consistent across library
- Python: import from chronoptics.tof instead of tof_chrono
- Python: removed tof_hidden, some of the functionality is moved to chronoptics.tof some is removed
- Python: now has type annotation
- C++: namespace is now chronoptics::tof instead of tof_chron
- C++: function naming convention is now snake_case consistently
- C++: removed error handling with templated class, now you have to define TOF_ERROR_HANDLER if you want to override default error handling
- Matlab: library is now in a namespace
- Matlab: now uses a single .mex file for all the functions and first argument is function name

Features:
- New programming language support is now relatively easy. Contact us if you have a request for an additional programming language
- Added support for user pointers
- Improved gpu performance
- Improved performance for computers with shared gpu/cpu memory (integrated graphics)
- Added amplitude dot segmentation filter, it's also quite effective at removing shadowing effects dut to the distance between light source depth sensor
- Armv8 release
- Added support for custom software on camera
- Added UserConfig this is a class that you can set simple question on and then generate a CameraConfig to your preferences without having to know all the difficult bits.
- Added ros node example
- Matlab: frames are now passed around without data copies

Viewer features:
- Added buffer to allow you to replay previously captured data
- Improved csf capture interface
- Display now starts mirrored and flipped such that it looks correct in default orientation
- Improved viewer performance

## Release 2.2.4
Features:
- Added illum phase delay to mlx75027 camera config
- Added on camera processing to KeaCamera

## Release 2.2.3
Features:
- Added illumination mode to camera config
- Added RAW_AB support for mlx sensor
Fixes:
- CMake ToFConfig is now located in lib/cmake/tof

## Release 2.2.2
Features:
- Added checksums to calibration and configuration files
- Added crash report functions
Fixes:
- Upper case serial numbers are now accepted.
- Fixed several usb issues

## Release 2.2.1
Fixes:
- Improved file upload functionality
- Fixed opencl issue on mac

## Release 2.2.0
Feature updates:
- Added USB camera support
- CSF file checksums are now checked

## Release 2.1.0
Feature updates:
- ToF library now supports RGB-D cameras. 
- Configurations for older cameras can now be generated and uploaded. 

## Release 2.0.1
Updated libraries used for building and now runs on mac

## Release 2.0.0 
Feature updates
- GPU pipeline on and off camera. 
- MixedPixel GPU working. 
- CameraConfig() class for configuring Kea revC cameras 
- Removed the need for tof_hidden from python and matlab for future camera release for KeaC
- Added viewer.exe as part of the release 
- Gige camera API rework 
- Kea camera API rework 
- Added blank camera, so can quickly add other cameras to the depth pipeline 
- Updated Matlab and Python examples
- Added more python, matlab and C++ test cases
- Gige far more reliable, start/stop test passing, along with find and discover 
- New camera config file version, so old camera releases will need to be updated

## Release 1.3.0
Fixed issues with streaming at higher data rates

## Release 1.2.1 
Feature updates
- Can stream depth data off camera. The following depth processing is supported 
    - Gaussian filtering, kernel size of 2, 3, and 5
    - Median filtering, kernel size of 3 and 5
    - Bilateral filtering, kernel size of 3 and 5
    - Temporal filtering
    - Flying pixel removal 
    - Phase unwrapping of two frequencies. This extends the maximum operating distance to the greatest common denominator. For example combining 40MHz and 50Mhz results in an effective frequency of 10MHz, and therefore a maximum distance of 15m. 
    - Mixed Pixel, can resolve multi-path interference using two modulation frequencies.  
- Configurable MIPI speed (can run faster frame rate at 600Mbps) 
- ROI, binning, flip and mirror works correctly in depth pipeline. 
- MixedPixel return selection. 

Bug Fixes
- Example OpenCV API viewer now compiles by default
- Intensity scaling now works correctly
- Detection of fifo overflow and reboots camera
- Binning of FPO when phase wrapping is present is now fixed. 
- ReadIn bug fixed. Found out if the camera stops during a MIPI transfer that frame data is not flushed from the MIPI RX buffer on the IMX8.

Examples
- Examples on how to generate camera configurations and upload them to the camera 
- Matlab, stream_frame_camera_single_freq.m streams depth using a single frequency 
- Matlab, stream_mixedpixel_from_camera.m streams mixed pixel output from camera 
    
Matlab Updates

Other Notes
- The GigeCamera() API is still under development for the Kea camera. 
